'
' RFID toy
' Read RFID embedded in wooden blocks and send a keystroke to the computer
' v0
'
' by Stelios M. (steliosm@gmail.com)
'
'
' RFID token IDs: 90E05F,AD23A1,7FD080,9DCE7C,183106,6CBFFC,53A8D4,4AC0A5,58CBBC,8176D8,7DD587,4CC8AB,74A9F2,
' 				6AFDB8,7AC396,4C7310,7A9FCA,7D7624,704D12,501A65,7AB2E7,16FEC7,4BB3D7,046A41,4C4023
'


'
' Changelog:
'	v0.5 Added a switch to select between GIDEI or keyboard scan codes.
'		 Added a small piezo to sound whenever data is send to the computer
'	v0.4 Added a cool down timer in order to send the same char after a while
'	v0.3 Switched to serout on port C.1 - also using 'T' mode to comply with CP2102 module
'	v0.2 Does not repeat character is token is left on the reader
'	v0.1 Added qualifier to the serin
'
'


' Move speed to 16MHz to handle RFID module serial in at 9600bps
setfreq m16

' Disconnect
'disconnect

' Only download the data table with the token IDs the first time or in every update
#no_data


' Define the variables
symbol COUNTER	   	= b23		 ' Store the counter value when looping
symbol CHAR		  	= b24		 ' Store the character result from table lookup
symbol POS		   	= b25		 ' Hold the value of the position to lookup the table
symbol OFFSET			= 6		   ' Size of RFID data - used to move forward while searching the EEPROM

' Define extra stuff
symbol READ_DELAY		= 1000		' Delay between reading from the RFID device
symbol COOL_DOWN_DELAY   = 3	   	' Delay in seconds before sending the same character
symbol COOL_DOWN_TIMER   = w13	 	' Store cool down timer value
symbol BREAK_KEY		 = $F0		 ' Keyboard scancode break value

' Define the pins
symbol DATA_OUT     	 = C.1		' Connection to the PC
symbol RFID_IN		   = C.2		' Connection to the RFID module
symbol DATA_MODE		 = pinC.3	 ' Connection to the mode switch
symbol PIEZO			 = C.4		' Connection to the piezo


' Define debug option
'#define debug

'
' Boot up
'
' Play a tone to let us know you are ready
sound PIEZO, (1, 50)
sound PIEZO, (1, 50)


' Init the Cool Down timer
let COOL_DOWN_TIMER = time + COOL_DOWN_DELAY
#ifdef debug
	sertxd ("Time: ", #time, " Cool down: ", #COOL_DOWN_TIMER, 10, 13)
#endif
	

' Main loop
main:
	' Read data in from RFID module
	' note:  Disc type rfid tags start with 560079
	serin RFID_IN, t9600_16, ("560079"), b0, b1, b2, b3, b4, b5
	#ifdef debug
		sertxd ("Got data!", 10, 13)
	#endif

	' Look up the data in EEPROM
	gosub lookup_key

	' Make a pause before reading again
	pause READ_DELAY 
goto main



'
' Read groups of characters from the EEPROM and sompare them with the characters received through the
' RFID reader. If a match is found, do a lookup to pick the character to send.
'
lookup_key:
	' Do a fast lookup using the currently avaiable values in b6 - b11
	if b0 = b6 and b1 = b7 and b2 = b8 and b3 = b9 and b4 = b10 and b5 = b11 then
		' Check if the Cool Down timer has expired
		if time < COOL_DOWN_TIMER then
			#ifdef debug
				sertxd("[", #time, "/", #COOL_DOWN_TIMER, "] Same card", 10, 13)
			#endif
	
			' Skip the lookup
			goto lookup_end
		else
			#ifdef debug
				sertxd("[", #time, "/", #COOL_DOWN_TIMER, "] Cool down expired", 10, 13)
			#endif
						
			' Init the Cool Down timer
			let COOL_DOWN_TIMER = time + COOL_DOWN_DELAY
			
			' Send the character
			goto lookup_send
		end if 
	end if


	' Scan the full EEPROM
	let POS = 0
	for COUNTER = 0 to 24
		' Define the position
		let POS = COUNTER * OFFSET
		
		' Read from EEPROM
		read POS, b6, b7, b8 ,b9 ,b10, b11
		#ifdef debug
			sertxd ("cnt: ", #COUNTER, ", token: ", b0, b1, b2, b3, b4, b5, ", data: ", b6, b7, b8, b9, b10, b11, 10, 13)
		#endif
			
		' Check if we have a match
		if b0 = b6 and b1 = b7 and b2 = b8 and b3 = b9 and b4 = b10 and b5 = b11 then
			#ifdef debug
				sertxd ("MATCH:  cnt: ", #COUNTER, ", token: ", b0, b1, b2, b3, b4, b5, ", data: ", b6, b7, b8, b9, b10, b11, 10, 13)
			#endif
		
			' We have a match, pick the right character from the lookup lists
			gosub lookup_value
			
			' Exit the loop
			exit
		end if
	next COUNTER
	
	lookup_send:
		' Send the keystroke
		' We are using the GIDEI standard, so we just send the character of each key
		' Send the data out using 9600 8,N,1
		#ifdef debug
			' Show debug information
			sertxd ("ID: ", b0, b1, b2, b3, b4, b5, ", char: ", CHAR, ", counter: ", #COUNTER, 10, 13)
		#endif

		' Play a note
		sound PIEZO, (1, 50)
		
		' Send the character out
		serout DATA_OUT, t9600_16, (CHAR)
		
		' Check if we need to send a 'break' sequence
		if DATA_MODE = 0 then
			' Send the 'Break' sequence
			serout DATA_OUT, t9600_16, (BREAK_KEY, CHAR)
		end if

	' Subroutine exit point
	lookup_end:
return


'
' Pick up the correct value based on the protocol selected
'
lookup_value:
	if DATA_MODE = 1 then
		' Look up table for GIDEI/AAC Keys format
		' https://park.org/Guests/Trace/pavilion/gideido1.htm
		lookup COUNTER, ("abgdezhuiklmnjoprstyfxcvw"), CHAR
	else
		' Support for keyboard scan codes
		' Works with inputattach 'ps2serkbd' mode
		lookup COUNTER, ($1C,$32,$43,$23,$24,$1A,$33,$3C,$43,$42,$4B,$3A,$31,$3B,$44,$4D,$2D,$1B,$2C,$35,$2B,$22,$21,$2A,$1D), CHAR
	end if

return


'
' Fill up the table with the token IDs
' The token IDs have been limited to 6 bytes per ID, which allows for aprox 42 ID to be stored.
' Currently using 25x6 = 150bytes
'
data 0, ("90E05FAD23A17FD0809DCE7C1831066CBFFC53A8D44AC0A558CBBC8176D87DD5874CC8AB74A9F26AFDB87AC3964C73107A9FCA7D7624704D12501A657AB2E716FEC74BB3D7046A414C4023")	' Put the RFID token IDs here

